//
//  SDKBridge.cpp
//  iOSWrapper
//
//  Created by Tabish on 11/15/15.
//  Copyright © 2015 AdscendMedia. All rights reserved.
//

#import "SDKBridge.h"
#import <AdscendMedia/AdscendMedia.h>

extern "C" UIViewController* UnityGetGLViewController();
extern "C" void UnitySendMessage(const char* obj, const char* method, const char* msg);

@interface SDKBridge () <ADOffersViewControllerDelegate, ADRewardedVideoViewControllerDelegate, ADMarketResearchVCDelegate>
{
    
}
@property (nonatomic, strong) NSString* callerName;
@end

@implementation SDKBridge

/*
 * Use of sdkInstance is to effectively call self inside C++ methods which otherwise you cannot do
 */
void initSDK () {
    if (!sdkInstance) {
        [[SDKBridge alloc] init];
    }
}

void _showOfferWall(char *objectName, char *publisherId, char *adwallId, char *subId1)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    [sdkInstance showOfferWall:[NSString stringWithUTF8String:publisherId]
                      adwallId:[NSString stringWithUTF8String:adwallId]
                        subId1:[NSString stringWithUTF8String:subId1]];
}

void _showOfferWallWithOptions(char *objectName, char *publisherId, char *adwallId, char *subId1, char *options)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    [sdkInstance showOfferWall:[NSString stringWithUTF8String:publisherId]
                      adwallId:[NSString stringWithUTF8String:adwallId]
                        subId1:[NSString stringWithUTF8String:subId1]
                        options:[NSString stringWithUTF8String:options]];
}

void _showMarketResearch(char *objectName, char *publisherId, char *profileId, char *subId1)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    [sdkInstance showMarketResearch:[NSString stringWithUTF8String:publisherId]
                      profileId:[NSString stringWithUTF8String:profileId]
                        subId1:[NSString stringWithUTF8String:subId1]];
}

void _showMarketResearchWithOptions(char *objectName, char *publisherId, char *profileId, char *subId1, char *options)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    [sdkInstance showMarketResearch:[NSString stringWithUTF8String:publisherId]
                          profileId:[NSString stringWithUTF8String:profileId]
                             subId1:[NSString stringWithUTF8String:subId1]
                            options:[NSString stringWithUTF8String:options]];
}

void _playRewardedVideo(char *objectName, char *publisherId, char *profileId, char *subId1)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    [sdkInstance playRewardedVideo:[NSString stringWithUTF8String:publisherId]
                         profileId:[NSString stringWithUTF8String:profileId]
                            subId1:[NSString stringWithUTF8String:subId1]];
}

//Wrapper Functions
void _getCompletedOffers(char *objectName, char *publisherId, char *adWallId, char *subId1)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    
    [AdscendMediaWrapper getCompletedOffersWithPubId: [NSString stringWithUTF8String:publisherId]
                                           profileId:[NSString stringWithUTF8String:adWallId]
                                              subId1:[NSString stringWithUTF8String:subId1]
                                             success:^(NSArray *completedOffers) {
                                                 if(completedOffers.count)
                                                 {
                                                     NSData *completedOfferData = [NSJSONSerialization dataWithJSONObject:completedOffers options:NSJSONWritingPrettyPrinted error:nil];
                                                     
                                                     NSString *finalStr = [[NSString alloc] initWithData:completedOfferData encoding: NSUTF8StringEncoding];
                                                     
                                                     UnitySendMessage([sdkInstance.callerName UTF8String], "getCompletedOffersResponseSuccess", [finalStr UTF8String]);
                                                 }
                                                 else
                                                 {
                                                     UnitySendMessage([sdkInstance.callerName UTF8String], "getCompletedOffersResponseSuccess", "");
                                                 }
                                             } failure:^{
                                                 UnitySendMessage([sdkInstance.callerName UTF8String], "getCompletedOffersResponseFailure", "");
                                             }];
}

void _getOfferList(char *objectName, char *publisherId, char *adWallId, char *subId1, int offset)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    
    [AdscendMediaWrapper getOfferList:[NSString stringWithUTF8String:publisherId]
                             adWallId:[NSString stringWithUTF8String:adWallId]
                               subId1:[NSString stringWithUTF8String:subId1]
                               offset:offset
                              success:^(NSArray *offerList, BOOL moreAvailable) {
                                  if(offerList.count)
                                  {
                                      NSString *offersJson = [sdkInstance getOffersDataForOffers:offerList moreAvailabe:moreAvailable];
                                      
                                      UnitySendMessage([sdkInstance.callerName UTF8String], "getOffersResponseSuccess", [offersJson UTF8String]);
                                  }
                                  else
                                  {
                                      UnitySendMessage([sdkInstance.callerName UTF8String], "getOffersResponseSuccess", "");
                                  }
                                  
                              } failure:^(NSString *) {
                                  UnitySendMessage([sdkInstance.callerName UTF8String], "getOffersResponseFailure", "");
                              }];
}

//@param productId - @"4" or @"10";
// @"4" is for surveys on offer wall profile
// @"10" is for surveys on mr profile
void _getSurveys(char *objectName, char *publisherId, char *profileId, char *subId1, char *productId)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    
    [AdscendMediaWrapper getSurveyListForPubId:[NSString stringWithUTF8String:publisherId]
                                     profileId:[NSString stringWithUTF8String:profileId]
                                        subId1:[NSString stringWithUTF8String:subId1]
                                     productId:[NSString stringWithUTF8String:productId] success:^(NSArray *surveyList) {
                                         
                                         if(surveyList.count)
                                         {
                                             NSData *surveysData = [NSJSONSerialization dataWithJSONObject:surveyList options:NSJSONWritingPrettyPrinted error:nil];
                                             
                                             NSString *finalStr = [[NSString alloc] initWithData:surveysData encoding: NSUTF8StringEncoding];
                                             
                                             UnitySendMessage([sdkInstance.callerName UTF8String], "getSurveysResponseSuccess", [finalStr UTF8String]);
                                         }
                                         else
                                         {
                                             UnitySendMessage([sdkInstance.callerName UTF8String], "getSurveysResponseSuccess", "");
                                         }
                                         
                                     } notFound:^{
                                         
                                         UnitySendMessage([sdkInstance.callerName UTF8String], "getSurveysResponseNotFound", "");
                                         
                                     } failure:^{
                                         UnitySendMessage([sdkInstance.callerName UTF8String], "getSurveysResponseFailure", "");
                                     }];
    
}

void _getTransactions(char *objectName, char *publisherId, char *adWallId, char *subId1)
{
    initSDK();
    sdkInstance.callerName = [NSString stringWithUTF8String:objectName];
    
    [AdscendMediaWrapper  getTransactionListForPubId:[NSString stringWithUTF8String:publisherId]
                                            adWallId:[NSString stringWithUTF8String:adWallId]
                                              subId1:[NSString stringWithUTF8String:subId1]
                                             success:^(NSArray *transactionList) {
                                                 if (transactionList.count)
                                                 {
                                                     NSData *transactionsData = [NSJSONSerialization dataWithJSONObject:transactionList options:NSJSONWritingPrettyPrinted error:nil];
                                                     
                                                     NSString *finalStr = [[NSString alloc] initWithData:transactionsData encoding: NSUTF8StringEncoding];
                                                     
                                                     UnitySendMessage([sdkInstance.callerName UTF8String], "getTransactionsResponseSuccess", [finalStr UTF8String]);
                                                 }
                                                 else
                                                 {
                                                     UnitySendMessage([sdkInstance.callerName UTF8String], "getTransactionsResponseSuccess", "");
                                                 }
                                             } failure:^{
                                                 UnitySendMessage([sdkInstance.callerName UTF8String], "getTransactionsResponseFailure", "");
                                             }];
}

# pragma mark - Obj-C Methods

# pragma mark Init

- (id) init {
    self = [super init];
    if (self) {
        sdkInstance = self;
    }
    return self;
}

- (NSString *) getOffersDataForOffers:(NSArray *) offerList moreAvailabe:(BOOL)moreAvailable
{
    NSDictionary *offerDict = @{@"offers" : offerList,
                                @"moreAvailable": [NSNumber numberWithBool:moreAvailable]};
    
    NSData *offerData = [NSJSONSerialization dataWithJSONObject:offerDict options:NSJSONWritingPrettyPrinted error:nil];
    
    NSString *offerJson = [[NSString alloc] initWithData:offerData encoding: NSUTF8StringEncoding];
    
    return  offerJson;
}

- (void) showOfferWall:(NSString*)publisherId adwallId:(NSString*)adwallId subId1:(NSString*)subId1 options:(NSString*)options
{
    NSLog(@" %@ %@ %@ %@", publisherId, adwallId, subId1, options);
    
    ADOffersViewController* offersVC = [ADOffersViewController unityOffersWallForPublisherId:publisherId adwallId:adwallId subId1:subId1 delegate:sdkInstance];
    
    NSArray *optionsArray = [options componentsSeparatedByString:@"\n"];
    for (int i=0; i < [optionsArray count]; i++)
    {
        NSString *keyValuePair = [optionsArray objectAtIndex:i];
        NSRange range = [keyValuePair rangeOfString:@"="];
        if (range.location != NSNotFound) {
            NSString *key = [keyValuePair substringToIndex:range.location];
            NSString *value = [keyValuePair substringFromIndex:range.location+1];
            
            if ([key isEqualToString:@"gender"])
            {
                offersVC.uGender = value;
            }
            else if ([key isEqualToString:@"age_range_id"])
            {
                offersVC.uAgeRange = value;
            }
            else if ([key isEqualToString:@"subid2"])
            {
                offersVC.subid2 = value;
            }
            else if ([key isEqualToString:@"subid3"])
            {
                offersVC.subid3 = value;
            }
            else if ([key isEqualToString:@"subid4"])
            {
                offersVC.subid4 = value;
            }
        }
    }
    
    [UnityGetGLViewController() presentViewController:offersVC animated:YES completion:nil];
}

- (void) showOfferWall:(NSString*)publisherId adwallId:(NSString*)adwallId subId1:(NSString*)subId1
{
    NSLog(@" %@ %@ %@", publisherId, adwallId, subId1);
    
    ADOffersViewController* offersVC = [ADOffersViewController unityOffersWallForPublisherId:publisherId adwallId:adwallId subId1:subId1 delegate:sdkInstance];
    
    [UnityGetGLViewController() presentViewController:offersVC animated:YES completion:nil];
}

- (void) showMarketResearch:(NSString*)publisherId profileId:(NSString*)profileId subId1:(NSString*)subId1 options:(NSString*)options
{
    NSLog(@" %@ %@ %@ %@", publisherId, profileId, subId1, options);
    
    ADMarketResearchViewController* marketResearchVC = [ADMarketResearchViewController unityMarketResearchForPublisherId:publisherId profileId:profileId subId1:subId1 delegate:sdkInstance];
    
    NSArray *optionsArray = [options componentsSeparatedByString:@"\n"];
    for (int i=0; i < [optionsArray count]; i++)
    {
        NSString *keyValuePair = [optionsArray objectAtIndex:i];
        NSRange range = [keyValuePair rangeOfString:@"="];
        if (range.location != NSNotFound) {
            NSString *key = [keyValuePair substringToIndex:range.location];
            NSString *value = [keyValuePair substringFromIndex:range.location+1];
            
            if ([key isEqualToString:@"subid2"])
            {
                marketResearchVC.subid2 = value;
            }
            else if ([key isEqualToString:@"subid3"])
            {
                marketResearchVC.subid3 = value;
            }
            else if ([key isEqualToString:@"subid4"])
            {
                marketResearchVC.subid4 = value;
            }
        }
    }
    
    [UnityGetGLViewController() presentViewController:marketResearchVC animated:YES completion:nil];
}

- (void) showMarketResearch:(NSString*)publisherId profileId:(NSString*)profileId subId1:(NSString*)subId1
{
    NSLog(@" %@ %@ %@", publisherId, profileId, subId1);
    
    ADMarketResearchViewController* marketResearchVC = [ADMarketResearchViewController unityMarketResearchForPublisherId:publisherId profileId:profileId subId1:subId1 delegate:sdkInstance];
    
    [UnityGetGLViewController() presentViewController:marketResearchVC animated:YES completion:nil];
}

- (void) playRewardedVideo:(NSString*)publisherId profileId:(NSString*)profileId subId1:(NSString*)subId1
{
    [ADRewardedVideoViewController unityRewardedVideoForPublisherId:publisherId profileId:profileId subId1:subId1 delegate:sdkInstance success:^(ADRewardedVideoViewController *rewardedVideoVC) {
        [UnityGetGLViewController() presentViewController:rewardedVideoVC animated:YES completion:nil];
    } failure:^(NSString *errorMsg) {
        UnitySendMessage([sdkInstance.callerName UTF8String], "onFailure", [errorMsg UTF8String]);
    }];
}

#pragma mark - ADMarketResearchVCDelegate
- (void)onCloseMarketResearchVCPressed
{
    NSLog(@"onCloseMarketResearchVCPressed");
    [UnityGetGLViewController() dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - ADOffersViewControllerDelegate
- (void)onCloseOffersVCPressed
{
    [UnityGetGLViewController() dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - ADRewardedVideoViewControllerDelegate
- (void)onUserCredited
{
    NSLog(@"onUserCredited");
    UnitySendMessage([sdkInstance.callerName UTF8String], "userWasCredited", "");
}

- (void)onUserNotCredited
{
    NSLog(@"onUserNOTCredited");
    UnitySendMessage([sdkInstance.callerName UTF8String], "userWasNotCredited", "");
}

- (void)onCloseVideoVCPressed
{
    NSLog(@"onCloseVideoVCPressed");
}

@end

